#ifndef CORE_VIEW_H
#define CORE_VIEW_H

#include "core/Matrix.hpp"

typedef struct {
    Matrix view;
    Vector3 back;
    Vector3 down;
    Vector3 front;
    Vector3 left;
    Vector3 right;
    Vector3 up;
} View;

void initView(View* v);
void updateDirections(View* v, float lengthAngle, float widthAngle);
void updateDirectionsQ(View* v, const Quaternion* q);
Matrix* updateMatrix(View* v, const Vector3* pos);

#include "core/math/Matrix.hpp"

namespace Core {
    class View final {
        Matrix view{};
        Vector3 right{};
        Vector3 up{};
        Vector3 back{};

    public:
        void updateDirections(float lengthAngle, float widthAngle);
        void updateDirections(const Quaternion& q);
        const Matrix& updateMatrix(const Vector3& pos);
        Vector3 getUp() const;
        Vector3 getDown() const;
        Vector3 getLeft() const;
        Vector3 getRight() const;
        Vector3 getFront() const;
        Vector3 getBack() const;
    };
}

#endif
