#include "core/Random.hpp"

static const size_t M = 7;

void initRandom(Random* r, u32 seed) {
    r->index = 0;
    for(size_t i = 0; i < ARRAY_LENGTH(r->data); i++) {
        r->data[i] = seed;
        seed = seed * 7 + 31;
    }
}

static void update(Random* r) {
    static const u32 map[2] = {0, 0x8EBF'D028};
    static const size_t LENGTH = ARRAY_LENGTH(r->data);
    for(size_t i = 0; i < LENGTH - M; i++) {
        r->data[i] = r->data[i + M] ^ (r->data[i] >> 1) ^ map[r->data[i] & 1];
    }
    for(size_t i = LENGTH - M; i < LENGTH; i++) {
        r->data[i] =
            r->data[i + (M - LENGTH)] ^ (r->data[i] >> 1) ^ map[r->data[i] & 1];
    }
    r->index = 0;
}

#define LIMIT(value, min, max) (min + value % (max - min))

static u32 next(Random* r) {
    if(r->index >= ARRAY_LENGTH(r->data)) {
        update(r);
    }
    u32 u = r->data[r->index++];
    u ^= (u << 7) & 0x2B5B'2500;
    u ^= (u << 15) & 0xDB8B'0000;
    u ^= (u >> 16);
    return u;
}

u32 randomU32(Random* r, u32 min, u32 exclusiveMax) {
    u32 u = next(r);
    return LIMIT(u, min, exclusiveMax);
}

i32 randomI32(Random* r, i32 min, i32 exclusiveMax) {
    i32 i = (i32)(next(r) >> 1);
    return LIMIT(i, min, exclusiveMax);
}

size_t randomSize(Random* r, size_t min, size_t exclusiveMax) {
    size_t s = next(r);
    return LIMIT(s, min, exclusiveMax);
}

bool randomBool(Random* r) {
    return next(r) & 1;
}

float randomFloat(Random* r) {
    u32 u = next(r);
    float f = (float)u / (float)0xFFFFFFF;
    return f >= 1.0f ? randomFloat(r) : f;
}
