#include "core/View.hpp"

#include "core/Generic.hpp"

void initView(View* v) {
    *v = (View){0};
}

void updateDirections(View* v, float lengthAngle, float widthAngle) {
    angles(&v->front, lengthAngle, widthAngle);

    cross(&v->right, &v->front, &V(0.0f, 1.0f, 0.0f));
    normalize(&v->right);

    cross(&v->up, &v->right, &v->front);
    normalize(&v->up);

    invert(&v->left, &v->right);
    invert(&v->back, &v->front);
    invert(&v->down, &v->up);
}

void updateDirectionsQ(View* v, const Quaternion* q) {
    mul(&v->up, q, &V(0.0f, 1.0f, 0.0f));
    mul(&v->back, q, &V(-1.0f, 0.0f, 0.0f));

    cross(&v->right, &v->up, &v->back);
    normalize(&v->right);

    invert(&v->left, &v->right);
    invert(&v->front, &v->back);
    invert(&v->down, &v->up);
}

Matrix* updateMatrix(View* v, const Vector3* pos) {
    Vector4* d = v->view.data;
    d[0] = V(v->right.x, v->right.y, v->right.z, -dot(&v->right, pos));
    d[1] = V(v->up.x, v->up.y, v->up.z, -dot(&v->up, pos));
    d[2] = V(v->back.x, v->back.y, v->back.z, -dot(&v->back, pos));
    d[3] = V(0.0f, 0.0f, 0.0f, 1.0f);
    return &v->view;
}
